import gzip
import shutil
import tempfile
import zipfile
from pathlib import Path
from distutils.dir_util import copy_tree


class BackupHandler:
    source_file_path = ''
    db_handler = None

    __debug = False

    def __init__(self, source_file_path, debug=False):
        self.source_file_path = source_file_path
        self.__debug = debug

    def __unzip_all_gz2sql(self, path):
        # unzip all .gz files to sql files
        self.__print('unzipping ' + str(path))

        for gz_file in Path(path).rglob('*.gz'):
            gz_file_path = Path(gz_file).absolute()
            self.__gunzip_shutil(gz_file, str(gz_file_path).replace('.gz', '.sql'))

    def __gunzip_shutil(self, source_filepath, destination_filepath, block_size=65536):
        # unzip a given source file
        self.__print('unzipping ' + str(source_filepath))

        with gzip.open(source_filepath, 'rb') as s_file, \
                open(destination_filepath, 'wb') as d_file:
            shutil.copyfileobj(s_file, d_file, block_size)

    # handle the given folder or zip file
    def resolve(self):
        self.__print('resolving the backup file...')

        temp_dir = tempfile.mkdtemp()
        sql_files_array = []

        if self.source_file_path.endswith('.zip'):
            zf = zipfile.ZipFile(self.source_file_path)
            zf.extractall(temp_dir)
        else:
            copy_tree(self.source_file_path, temp_dir)

        self.__unzip_all_gz2sql(temp_dir)
        for file in Path(temp_dir).rglob('*.sql'):
            sql_files_array.append(str(file))

        return temp_dir, sql_files_array

    def __print(self, msg):
        if self.__debug:
            print(msg)



import shutil
from os import path
from resurrector.db_handler import DBHandler
from resurrector.backup_resolver import BackupHandler


def resurrect(source, host='localhost', user='root', password='', port=3306, skip_dbs=None, skip_existing=True,
              mysql_global_vars=None,
              exclude_sql_parts=None,
              char_set='utf8', collate='utf8_general_ci', debug=False):
    tmp_dir = ''
    try:
        tmp_dir, sql_files = BackupHandler(source, debug=debug).resolve()
        db = DBHandler(host, user, password, port,
                       skip_dbs, skip_existing, mysql_global_vars, exclude_sql_parts,
                       char_set, collate, debug)
        db.restoreMultiple(sql_files)
    except Exception as e:
        print('Resurrector failed the resurrection of a database. Error | ', e)
    finally:
        if path.isdir(tmp_dir):
            shutil.rmtree(tmp_dir)



# provides different functions such as :
# creating a database
# filling a database

import os
import MySQLdb


class DBHandler:
    __host = None
    __user = None
    __password = None
    __port = None

    __char_set = 'utf8'
    __collate = 'utf8_general_ci'

    __exclude_sql_parts = ['SET @@GLOBAL.GTID_PURGED='';']
    __mysql_global_vars = ['SET GLOBAL max_allowed_packet=268435456;']  # for admin
    __skip_dbs = []
    __skip_existing = True

    __existing_databases = []

    __debug = False

    def __init__(self, host, user, password, port,
                 skip_dbs, skip_existing, exclude_sql_parts, mysql_global_vars,
                 char_set, collate, debug):
        self.__host = host
        self.__user = user
        self.__password = password
        self.__port = port
        self.__skip_existing = skip_existing

        if self.__skip_existing is True:
            self.__existing_databases = self.__getExisting()

        if skip_dbs is not None:
            self.__skip_dbs = self.__skip_dbs + skip_dbs

        if exclude_sql_parts is not None:
            self.__exclude_sql_parts = self.__exclude_sql_parts + exclude_sql_parts

        if mysql_global_vars is not None:
            self.__mysql_global_vars = self.__mysql_global_vars + mysql_global_vars

        self.__char_set = char_set
        self.__collate = collate

        self.__debug = debug

        self.__print('db handler initialized...')

    def __getExisting(self):

        self.__print('fetching existing schemas...')

        list_dbs_query = 'SHOW SCHEMAS;'
        conn = self.__getConnection()
        cur = conn.cursor()
        cur.execute(list_dbs_query)
        result = cur.fetchall()

        self.__print(str(len(result)) + ' schemas found...')

        dbs = []
        for r in result:
            dbs.append(r[0])

        return dbs

    def __getConnection(self, db=None):
        if db is not None:
            self.__print('connecting to ' + self.__host + ' ' + db + ' ...')
        else:
            self.__print('connecting to ' + self.__host)
        if db is not None:
            return MySQLdb.connect(host=self.__host,
                                   user=self.__user,
                                   passwd=self.__password,
                                   port=int(self.__port),
                                   db=db)
        else:
            return MySQLdb.connect(host=self.__host,
                                   user=self.__user,
                                   passwd=self.__password,
                                   port=int(self.__port))

    def __createDB(self, db_name):
        try:
            self.__print('creating db ' + db_name + ' ...')

            create_db_query = "CREATE DATABASE IF NOT EXISTS {db} CHARACTER SET {chs} COLLATE {cl};".format(
                db=db_name, chs=self.__char_set, cl=self.__collate)

            conn = self.__getConnection()
            cur = conn.cursor()
            cur.execute(create_db_query)
            conn.close()

            return True
        except Exception as e:
            # not created!
            return False

    def restoreMultiple(self, paths):
        for path in paths:
            db_name = self.__createDBName(path)
            if db_name in self.__skip_dbs:
                self.__print('skipping ' + db_name + ' because was set to be skip...')
                continue
            if db_name in self.__existing_databases:
                self.__print('skipping ' + db_name + ' because it already exists...')
                continue
            if self.__createDB(db_name):
                self.__restoreSingle(db_name, self.__get_content(path))

    def __restoreSingle(self, db_name, sql_content):

        self.__print('removing unnecessary content...')
        for pt in self.__exclude_sql_parts:
            sql_content = sql_content.replace(pt, "")

        # todo generalize this line
        sql_content = sql_content.encode('utf-8')
        try:
            self.__print('restoring ' + db_name + '...')

            conn = self.__getConnection(db_name)
            cursor = conn.cursor()
            cursor.execute(' '.join(self.__mysql_global_vars))
            cursor = conn.cursor()
            cursor.execute(sql_content)
            conn.close()
            return True
        except (MySQLdb.Error, MySQLdb.Warning) as e:
            self.__print(e)
            return False

    def __get_content(self, path):
        self.__print('getting content from ' + path + '...')
        sql_content = ""
        with open(path, "r", encoding='utf-8') as fh:
            for line in fh:
                sql_content += line
                pass
        return sql_content

    def __createDBName(self, path):
        return os.path.basename(path).replace('.sql', '')

    def __print(self, msg):
        if self.__debug:
            print(msg)



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.7'
DESCRIPTION = 'Restoring MySQL Databases from their backups'
LONG_DESCRIPTION = 'Resurrector helps you to restore your backed-up MySQL Databases to any specified instance.'

# Setting up
setup(
    name="resurrector",
    version=VERSION,
    author="rexshijaku (Rexhep Shijaku)",
    author_email="<rexhepshijaku@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=['resurrector'],
    install_requires=[],
    keywords=['python', 'mysql', 'database', 'backup', 'restore'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)